/* Toolbar FontAwesome icons
 * Need to override 'color: unset !important':
 * https://github.com/jupyterlab/jupyterlab/blob/v2.0.1/packages/ui-components/style/base.css#L70-L73
 */
.jp-OfflineNotebookToolbarIcon {
  color: var(--jp-inverse-layout-color3);
}
